""" HSUVision Class
"""
import sys
import numpy
from .params import *
from .msgs import *
from .contours import *


class HSUVision:
    """ There are three ways to use pre-defined comparable contours
        1. Use default saved contours from param
        Contours are Dictionaries, key is the letter name, value is the saved contour
        For custom contours one of two mentioned methods should be used as value in dictionary
        2. Use a string path to .npy files (Only use files which generated by saveSampleContours.py script
        3. Numpy ndarrays with (n,1,2) shape (n is the number of contour points)
    """

    def __init__(self, contours_dict= {'H': h_cnt, 'S': s_cnt, 'U': u_cnt}):
        if not isinstance(contours_dict, dict):
            raise TypeError(ref_contours_type_error)
        _cnts_dict = contours_dict.copy()
        for name, contour in _cnts_dict.items():
            if not ((isinstance(contour, str) and contour[-4:] == '.npy') or isinstance(contour, numpy.ndarray)):
                raise TypeError(ref_contour_type_error)
            elif isinstance(contour, str) and contour[-4:] == '.npy':
                _cnts_dict[name] = numpy.load(contour)
        self.ref_contours = _cnts_dict.copy()

    def find_HSU(self, img, verbose=False):
        return find_HSU(img, verbose, ref_contours=self.ref_contours)



""" This is an example code for generating new contours"""
import cv2 as cv
from numpy import save
import contours

# Read a sample image from file
img = cv.imread('../test data/u.jpg')
# Find Image Contour
cnt, center = contours.find_contour(img, verbose=False)
# Save sample Contour to a .npy file
save('../Contours/U_contour.npy', cnt)
# Draw and show sample Contour
cv.drawContours(img, [cnt], 0, (0, 255, 0), 1)
cv.imshow('result', img)
cv.waitKey()



import numpy as np
import cv2 as cv

from .params import *

# Finding the letter contour in the picture
def find_contour(img, verbose):
    img_gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
    ret, thresh = cv.threshold(img_gray, cv_lower_thr, cv_upper_thr, 0)
    contours, hierarchy = cv.findContours(thresh, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
    # Return None if there is no contour in picture
    if len(contours) == 0:
        return None, None
    areas = [cv.contourArea(cnt) for cnt in contours]
    if verbose:
        print('         ... Containing areas are:', areas)
    if not np.any(np.logical_and(np.array(areas) < area_upper_bound, np.array(areas) > area_lower_bound)):
        return None, None

    cnt = contours[np.argmin(areas)]
    x, y, w, h = cv.boundingRect(cnt)
    center = [x + w / 2, y + h / 2]
    shifted_cnt = cnt - [x, y]
    scaled_cnt = shifted_cnt
    scaled_cnt[:, :, 0] = np.uint(shifted_cnt[:, :, 0] * scaled_cnt_coef / w)
    scaled_cnt[:, :, 1] = np.uint(shifted_cnt[:, :, 1] * scaled_cnt_coef / h)
    return scaled_cnt, center


# Find difference of contours
def find_diff(cnt1, cnt2):
    return cv.matchShapes(cnt1, cnt2, match_alg, 0)


# Find the HSU Letter in contuor
def find_HSU(img, verbose, ref_contours):
    contour, center = find_contour(img, verbose)
    if contour is None:
        return None, None
    min_diff = 10*match_thr
    extracted_ref = None
    for name, ref_cnt in ref_contours.items():
        diff = find_diff(contour, ref_cnt)
        if diff < min_diff:
            min_diff = diff
            extracted_ref = name
    # debug
    # cv.drawContours(img,[h_cnt],0,(0,255,0),1)
    # cv.drawContours(img, [contour], 0, (0, 0, 255), 1)
    # cv.imshow('c',img)
    # cv.waitKey()
    if min_diff > match_thr:
        return None, None
    if verbose:
        print('         ... diff to ref is:', min_diff)
    return extracted_ref, center



"""Setup module."""

from setuptools import setup, find_packages
from os import path


def get_requires():
    """Read requirements.txt."""
    requirements = open("requirements.txt", "r").read()
    return list(filter(lambda x: x != "", requirements.split()))


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md')) as f:
    long_description = f.read()

setup(
    # Name of Project
    # $ pip install RCJRVision
    # where it will live on PyPI: https://pypi.org/project/RCJRVision/
    name='RCJRVision',
    version='1.5',
    description='A fast and simple image processing method to detect H S U victims in rescue maze',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mhmmdshirazi/RCJRVision',
    author='Mohammad Mahdi Shirazi',
    author_email='mhmmdshirazi@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    packages=['RCJRVision'],
    python_requires='>=3.5, <4',
    install_requires=get_requires(),
)
